/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  ctransf.c                                                */
/*                                                           */
/* Latest update: Oct 95 SS, Aug 98                          */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#define swap(a,b) {temp=(a);(a)=(b);(b)=temp;}

double pi=3.14159265358979323;


void gaussj(a,n,b,m,indxc,indxr,ipiv)
double **a;
int n;
double **b;
int m;
int *indxc;
int *indxr;
int *ipiv;
{
   int i,icol,irow,j,k,l,ll;
   double big,dum,pivinv,temp;

   for (j=1; j<=n; j++) {
       ipiv[j]=0;
   } /* for j */

   for (i=1; i<=n; i++) {
      big=0.0;
      for (j=1; j<=n; j++) {
         if (ipiv[j] != 1) {
            for (k=1; k<=n; k++) {
               if (ipiv[k] == 0) {
                  if (fabs(a[j][k]) >= big) {
                     big=fabs(a[j][k]);
                     irow=j;
                     icol=k;
                  } /* if */
               }
               else if (ipiv[k] > 1) {
                    printf("Singular matrix 1\n");
                    exit(0);
               } /* else */
            } /* for k */
         } /* if */
      } /* for j */
      ipiv[icol]=ipiv[icol]+1;
      if (irow != icol) {
         for (l=1; l<=n; l++) {
             swap(a[irow][l],a[icol][l]);
         } /* for l */
         for (l=1; l<=m; l++) {
             swap(b[irow][l],b[icol][l]);
         } /* for l */
      } /* if */
      indxr[i]=irow;
      indxc[i]=icol;
      if (a[icol][icol] == 0.0) {
         printf("Singular matrix 2\n");
         exit(0);
      } /* if */
      pivinv=1.0/a[icol][icol];
      a[icol][icol]=1.0;
      for (l=1; l<=n; l++) {
          a[icol][l]=pivinv*a[icol][l];
      } /* for l */
      for (l=1; l<=m; l++) {
          b[icol][l]=pivinv*b[icol][l];
      } /* for l */
      for (ll=1; ll<=n; ll++) {
          if (ll != icol) {
             dum=a[ll][icol];
             a[ll][icol]=0.0;
             for (l=1; l<=n; l++) {
                 a[ll][l]=a[ll][l]-a[icol][l]*dum;
             } /* for l */
             for (l=1; l<=m; l++) {
                 b[ll][l]=b[ll][l]-b[icol][l]*dum;
             } /* for l */
          } /* if */
      } /* for ll */
   } /* for i */
  
   for (l=n; l>=1; l--) {
       if (indxr[l] != indxc[l]) {
          for (k=1; k<=n; k++) {
              swap(a[k][indxr[l]],a[k][indxc[l]]);
          } /* for k */
       } /* if */
   } /* for l */

} /* gaussj */
   

int main(argc,argv)
int argc;
char *argv[];

{
   FILE *infile1,*infile2,*cfile;

   int i,j,k,n1,n2,ixmax,iymax;

   int naa,maa,n0,na,m0;

   double x,y,dx,dy,inte;
   double inte1,inte2;
   double temp,sum,xmean,ymean,xssum,yssum;
   double xsigma,ysigma,xmax,ymax;

   double ax2,ax1,ax0,ay2,ay1,ay0;

   double *x1,*y1,*int1,*x2,*y2,*int2;

   double **aa,**aat;
   double *mpaa, *mpaat;

   double **aaa;
   double  *mpaaa;

   double **bbb;
   double  *mpbbb;

   double *bb2;

   int *t1,*t2,*t3;


   if (argc < 3) {
      printf("Usage: %s file1(x y int) file2(x y int) coeffile\n",argv[0]);
      exit(0);
   } /* if */ 



   if ((infile1 = fopen(argv[2],"r")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   if ((infile2 = fopen(argv[1],"r")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   cfile=fopen(argv[3],"w");

   i=1;

   while (fscanf(infile1,"%lf %lf %lf",&x,&y,&inte) != EOF) {
      i++;
   } /* while */

   n1=i-1;

   i=1;

   while (fscanf(infile2,"%lf %lf %lf",&x,&y,&inte) != EOF) {
      i++;
   } /* while */

   n2=i-1;


   x1=(double *)calloc(n1+2,sizeof(double));
   y1=(double *)calloc(n1+2,sizeof(double));
   int1=(double *)calloc(n1+2,sizeof(double));

   x2=(double *)calloc(n2+2,sizeof(double));
   y2=(double *)calloc(n2+2,sizeof(double));
   int2=(double *)calloc(n2+2,sizeof(double));


   rewind(infile1);

   i=1;

   while (fscanf(infile1,"%lf %lf %lf",&x,&y,&inte) != EOF) {
      x1[i]=x;
      y1[i]=y;
      int1[i]=inte;
      i++;
   } /* while */

   rewind(infile2);

   i=1;

   while (fscanf(infile2,"%lf %lf %lf",&x,&y,&inte) != EOF) {
      x2[i]=x;
      y2[i]=y;
      int2[i]=inte;
      i++;
   } /* while */


naa=3;
maa=n1;

t1=(int *)calloc(naa+1,sizeof(int));
t2=(int *)calloc(naa+1,sizeof(int));
t3=(int *)calloc(naa+1,sizeof(int));

aaa=(double **)calloc(naa+1,sizeof(double*));
mpaaa=(double *)calloc((naa+1)*(naa+1),sizeof(double));

for (i=0; i<=naa; i++) {
   aaa[i]=mpaaa;
   mpaaa=mpaaa+naa+1;
} /* for i */

bbb=(double **)calloc(naa+1,sizeof(double*));
mpbbb=(double *)calloc((naa+1)*2,sizeof(double));

for (i=0; i<=naa; i++) {
   bbb[i]=mpbbb;
   mpbbb=mpbbb+2;
} /* for i */

aa=(double **)calloc(maa+1,sizeof(double*));
aat=(double **)calloc(naa+1,sizeof(double*));

mpaa=(double *)calloc((maa+1)*(naa+1),sizeof(double));
mpaat=(double *)calloc((maa+1)*(naa+1),sizeof(double));

for (i=0; i<=maa; i++) {
   aa[i]=mpaa;
   mpaa=mpaa+naa+1;
} /* for i */

for (i=0; i<=naa; i++) {
   aat[i]=mpaat; 
   mpaat=mpaat+maa+1;
} /* for i */

bb2=(double *)calloc(maa+1,sizeof(double));


   for (k=1; k<=n1; k++) {
      temp=x2[k]; aa[k][3]=temp; aat[3][k]=temp;
      temp=y2[k]; aa[k][2]=temp; aat[2][k]=temp;
      temp=1.0; aa[k][1]=temp; aat[1][k]=temp;
      bb2[k]=x1[k];
   } /* for k*/

   n0=3;
   na=n1;    

   for (i=1; i<=n0; i++) {
      for (j=1; j<=n0; j++) {
         sum=0.0;
         for (k=1; k<=na; k++) {
            sum=sum+aat[i][k]*aa[k][j];
         } /* for k */
         aaa[i][j]=sum;
      } /* for j */
   } /* for i */ 


   for (j=1; j<=n0; j++) {
      sum=0.0;
      for (k=1; k<=na; k++) {
         sum=sum+aat[j][k]*bb2[k];
      } /* for k */
      bbb[j][1]=sum;
   } /* for j */


   m0=1;

   gaussj(aaa,n0,bbb,m0,t1,t2,t3);

   
   for (i=3; i>=1; i--) {
       fprintf(cfile,"%18.15f\n",bbb[i][1]);
   } /* for i */

   ax2=bbb[3][1];
   ax1=bbb[2][1];
   ax0=bbb[1][1];




   for (k=1; k<=n1; k++) {
      temp=x2[k]; aa[k][3]=temp; aat[3][k]=temp;
      temp=y2[k]; aa[k][2]=temp; aat[2][k]=temp;
      temp=1.0; aa[k][1]=temp; aat[1][k]=temp;
      bb2[k]=y1[k];
   } /* for k*/

   n0=3;
   na=n1;    

   for (i=1; i<=n0; i++) {
      for (j=1; j<=n0; j++) {
         sum=0.0;
         for (k=1; k<=na; k++) {
            sum=sum+aat[i][k]*aa[k][j];
         } /* for k */
         aaa[i][j]=sum;
      } /* for j */
   } /* for i */ 


   for (j=1; j<=n0; j++) {
      sum=0.0;
      for (k=1; k<=na; k++) {
         sum=sum+aat[j][k]*bb2[k];
      } /* for k */
      bbb[j][1]=sum;
   } /* for j */


   m0=1;

   gaussj(aaa,n0,bbb,m0,t1,t2,t3);

   fprintf(cfile,"\n");
   
   for (i=3; i>=1; i--) {
       fprintf(cfile,"%18.15f\n",bbb[i][1]);
   } /* for i */

   ay2=bbb[3][1];
   ay1=bbb[2][1];
   ay0=bbb[1][1];



   ixmax=0;
   iymax=0;
   xmax=-10000.0;
   ymax=-10000.0;
   xmean=0.0;
   ymean=0.0;   

   for (i=1; i<=n1; i++) {
      dx=x1[i]-ax2*x2[i]-ax1*y2[i]-ax0;
      dy=y1[i]-ay2*x2[i]-ay1*y2[i]-ay0;
      if (fabs(dx) > xmax) {
         xmax=fabs(dx);
         ixmax=i;
      } /* if */
      if (fabs(dy) > ymax) {
         ymax=fabs(dy);
         iymax=i;
      } /* if */
      xmean=xmean+dx;
      ymean=ymean+dy;
   } /* for i */

   xmean=xmean/(double)(n1);
   ymean=ymean/(double)(n1);
    
   xssum=0.0;
   yssum=0.0;

   fprintf(cfile,"\n");

   for (i=1; i<=n1; i++) {
      dx=x1[i]-ax2*x2[i]-ax1*y2[i]-ax0;
      dy=y1[i]-ay2*x2[i]-ay1*y2[i]-ay0;
      temp=dx-xmean;
      xssum=xssum+temp*temp;
      temp=dy-ymean;
      yssum=yssum+temp*temp;      
      fprintf(cfile,"%6d %9.3f %9.3f\n",i,dx,dy);
   } /* for i */


   xsigma=sqrt(xssum/(double)(n1-1));
   ysigma=sqrt(yssum/(double)(n1-1));

   fprintf(cfile,"\n");
   fprintf(cfile,"%16.3f %9.3f\n",xmean,ymean);
   fprintf(cfile,"%16.3f %9.3f\n",xsigma,ysigma);

   fprintf(cfile,"\n");
   fprintf(cfile,"%6d %9.3f\n",ixmax,xmax);
   fprintf(cfile,"%6d %9.3f\n",iymax,ymax);

} /* end */
