/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  fff2imf.c   SS April 97, Dec 97                          */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "image.h"


int main(argc,argv) 
int argc;
char *argv[];
{

   FILE *infile;

   struct image_head header; 
   short **image;
   
   int i,j,k,xsize,ysize,ll;

   double amp,max,amp0,temp;

   int xys[3];

   float *d;

   if (argc<3) {
      printf("Usage: %s image.fff image ampr\n",argv[0]);
      exit(0);
   } /* if */

   if ((infile = fopen(argv[1],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */


   amp0=atof(argv[3]);

   fread(&xys[1],sizeof(int),2,infile);

   xsize=xys[1];
   ysize=xys[2];

   ll=xsize*ysize;

   d=(float *)calloc(ll+1,sizeof(float));

   fread(&d[1],sizeof(float),ll,infile);

   fclose(infile);

   header.id=image_magic;
   header.typ=im_s16;
   header.off=512;
   header.hgh=ysize;
   header.wid=xsize;

   image = (short **)calloc(header.hgh,sizeof(short*));

   CreateIm_s16(image,header.hgh,header.wid);

   if (amp0 > 0.0) {
      amp=amp0;
   }
   else {

      max=-1.0e30;

      k=1; 
      for (i=0; i<xsize; i++) {
         for (j=0; j<ysize; j++) {
            if (d[k] > max) {
	        max=d[k];
	     } /* if */
	     k++;
         } /* for j */
      } /* for i */

      amp=32767.0/max;
   } /* if */


   k=1;
   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {
          temp=amp*d[k];
	  if (temp > 32767.0) {
	     temp=32767.0;
	  } /* if */
	  image[i][j]=(int)(temp+0.5);
	  k++;
      } /* for j */
   } /* for i */


   WriteImage(argv[2],image,header,im_s16);


} /* End  */
