/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*   frebind.c    SS April 94                                */
/*                                                           */
/*   Single precission.                                      */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include "image.h"


double pi=3.14159265358979323846264338;

double timetic=(double)(CLOCKS_PER_SEC);


int main(argc,argv) 
int argc;
char *argv[];

{

   struct image_head header;      
   struct image_head header2;
   short **image;
   short **image2;


   double size,invsize,eps,amp0,max,diffsec,sum,ftemp,ssize,ampr;

   int ss[3];

   int i,j,k,l,width,height,k2;
   int ndim,le,i2,j2,length,maxwh;
   int isign,tt,bfi,width2,height2,length2,off;
 

   char name[80];
   char outname[80];
 
   clock_t time1,timediff;

   float *dc,*dc2,*w;


if (argc < 4) {
   printf("Usage: %s inimage outimage binfactori ampr\n",argv[0]);
   exit(0);
} /* if */


strcpy(name,argv[1]);
strcpy(outname,argv[2]);
bfi=atoi(argv[3]);
ampr=atof(argv[4]);


ReadHeader(name,&header);

image = (short **)calloc(header.hgh,sizeof(long));

ReadImage(name,image);



width=header.wid;
height=header.hgh;

length=width*height;

width2=width/bfi;
height2=height/bfi;


header2.id=image_magic;
header2.typ=im_s16;
header2.off=512;
header2.hgh=height2;
header2.wid=width2;

image2 = (short **)calloc(header2.hgh,sizeof(long));

CreateIm_s16(image2,header2.hgh,header2.wid);



length2=width2*height2;

if (width2 > height2) {
   maxwh=width2;
}
else {
   maxwh=height2;
}

dc = (float *)calloc(2*length+1,sizeof(float));
dc2 = (float *)calloc(2*length2+1,sizeof(float));
w = (float *)calloc(2*maxwh+1,sizeof(float));

ampr=ampr/(double)(bfi*bfi);

size=(double)(width2*height2);
invsize=1.0/size;


k=1;
for (i=0; i<width; i++) {
   for (j=0; j<height; j++) {
      dc[k]=invsize*ampr*(double)(image[j][i]);
      k=k+2;
   } /* for j */
} /* for i */


 
   time1=clock();

   ndim=2;
   ss[1]=height;
   ss[2]=width;

   tt=0;
   isign=-1;

   dfourt_(&dc[1],&ss[1],&ndim,&isign,&tt,&w[1]);

   k=1;
 
   for (i=1; i<=2*length2; i++) {
      dc2[i]=0.0;
   } /* for i */

   k=1;
   k2=1;
   for (i=0; i<width2; i++) {
      for (j=0; j<height2; j++) {
         dc2[k2]=dc[k];
         dc2[k2+1]=dc[k+1];
         k=k+2;
         k2=k2+2;
         if (j==(height2-2)/2) {
           k=k+2*(bfi-1)*height2;
         }
      } /* for j */
      if (i==(width2-2)/2) {
         k=k+2*(bfi-1)*bfi*height2*width2;
      }
   } /* for i */


   ndim=2;
   ss[1]=height2;
   ss[2]=width2;

   tt=1;
   isign=1;

   dfourt_(&dc2[1],&ss[1],&ndim,&isign,&tt,&w[1]);

      k=1;
      for (i=0; i<width2; i++) {
         for(j=0; j<height2; j++) {
            image2[j][i]=(int)(dc2[k]+0.5);
            k=k+2;
         } /* for j */
      } /* for i */

   timediff=clock()-time1;

   printf("%10.2f\n",(double)(timediff)/timetic);

   WriteImage(outname,image2,header2,im_s16);

} /* End  */
