/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  imdot4.c   SS Oct 95                                     */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "image.h"

double pi=3.14159265358979323;

int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head header; 
   short **image;

   FILE *infile;

   int i,j,k,col,xi,yi,ord,s,xs,ys,dot;

   double x,y,mag,r,r0,rw,pi3,v,vs,x0,y0,rr;
   double dcol;


   if (argc<5) {
printf("Usage: %s inimage outimage file(x y int) rr rwr\n",argv[0]); 
      exit(0);
   } /* if */

   ReadHeader(argv[1],&header);

   image = (short **)calloc(header.hgh,sizeof(short*));

   ReadImage(argv[1],image);

   xs=header.wid-1;
   ys=header.hgh-1; 


   if ((infile = fopen(argv[3],"r")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   rr=atof(argv[4]);
   rw=atof(argv[5]);

   dot=1;
   ord=(int)(rr+0.5);

   pi3=2.0*pi+0.01*pi;

   s=(ord-1)/2;

 while (fscanf(infile,"%lf %lf %lf",&x,&y,&mag) != EOF) {

      xi=(int)(x+0.5);
      yi=(int)(y+0.5);
      dcol=0.0;
      k=0;

      x0=x;
      y0=y;
      r=rr; 
      r0=rr+rw;

      while (r <= r0) {
         if (r < 0.5) {
	    vs=pi;
	 }
	 else {
	    vs=pi/(20.0*r);
	 } /* if */
         v=0.0;
         while (v <= pi3) {
            xi=(int)(r*cos(v)+x0+0.5);
            yi=(int)(r*sin(v)+y0+0.5);
            if ((xi >= 0) && (xi < xs)) {
               if ((yi >= 0) && (yi < ys)) {
                  dcol=dcol+(double)(image[yi][xi]);
		  k++;
               } /* if */
            } /* if */
            v=v+vs;
         } /* while */
         r=r+0.5;
       } /* while */


      dcol=dcol/(double)(k);

      if (dcol >= 16384.0) {
         col=(int)(dcol-5000.0+0.5);
      }
      else {
         col=(int)(dcol+5000.0+0.5);
      } /* if */

      x0=x;
      y0=y;
      r=rr; 
      r0=rr+rw;

      while (r <= r0) {
         if (r < 0.5) {
	    vs=pi;
	 }
	 else {
	    vs=pi/(20.0*r);
	 } /* if */
         v=0.0;
         while (v <= pi3) {
            xi=(int)(r*cos(v)+x0+0.5);
            yi=(int)(r*sin(v)+y0+0.5);
            if ((xi >= 0) && (xi < xs)) {
               if ((yi >= 0) && (yi < ys)) {
                  image[yi][xi]=col;
               } /* if */
            } /* if */
            v=v+vs;
         } /* while */
         r=r+0.5;
       } /* while */

} /* while */
 


WriteImage(argv[2],image,header,im_s16);

} /* End  */
