/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  magnoise.c  SS Sept 96                                   */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h> 
#include <stdlib.h>
#include <unistd.h>

int main(argc,argv) 
int argc;
char *argv[];
{
   FILE *outfile;

   double x,y,psum,sum,m1,m2,ms,ga,bg,re;
   double fwhm,beta,gamma,temp,pf;
   double mag,lt,inte,pp;
   double sigma,dm,di,ee,a,c,eps;
   double alfa,r,mageps;

   int i,j,s;



   if (argc<9) {
      printf("Usage: %s m1r m2r mstepr fwhmr betar\n",argv[0]);
      printf("       gainfr bgr renr outfile\n"); 
      exit(0);
   } /* if */

   
   m1=atof(argv[1]);
   m2=atof(argv[2]);
   ms=atof(argv[3]);
   fwhm=atof(argv[4]);
   beta=atof(argv[5]);
   ga=atof(argv[6]);
   bg=atof(argv[7]);
   re=atof(argv[8]);

   outfile=fopen(argv[9],"w");

   lt=log(10.0);
   eps=1.0e-4;

   gamma=4.0*(exp(log(2.0)/beta)-1.0);
   alfa=sqrt((exp(log(eps)/(1.0-beta))-1.0)/gamma);

   r=fwhm*alfa;

   s=(int)(r+1.5);

   a=gamma/(fwhm*fwhm);
   c=gamma/(fwhm*fwhm);

   psum=0.0;

   for (i=-s; i<=s; i++) {
      for (j=-s; j<=s; j++) {
         x=(double)(i);
         y=(double)(j);
         temp=exp(-beta*log(1.0+a*x*x+c*y*y));
         psum=psum+temp; 
      } /* for j */
   } /* for i */  

   pf=1.0/psum;

   bg=ga*bg;
   re=ga*re;

   mag=m1;
   mageps=0.00001;

   while ((mag-mageps) <= m2) {
 
      inte=exp(-0.4*lt*mag);

      ee=ga*inte;

      sum=0.0;

      for (i=-s; i<=s; i++) {
         for (j=-s; j<=s; j++) {
            x=(double)(i);
            y=(double)(j);
            pp=pf*exp(-beta*log(1.0+a*x*x+c*y*y));
            sum=sum+(pp*pp)/(pp*ee+bg+re*re);
         } /* for j */
      } /* for i */  

      sigma=sqrt(1.0/sum);

      di=sigma/ga;

      dm=-2.5*log(1.0+di/inte)/lt;

      fprintf(outfile,"%9.3f %9.3f %12.3f %12.3f\n",mag,dm,inte,di);

      mag=mag+ms;

   } /* while */

   printf("      mag      dmag          int    sigma_int\n");
   printf("\n");
   printf("      Printed to file: %s\n",argv[9]);

} /* End  */
