/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*   medianh.c  SS July 92                                   */ 
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <stdlib.h>
#include "image.h"

double timetic=(double)(CLOCKS_PER_SEC);

void qqsort(f,sr,sl,n)
int *f;
int *sr;
int *sl;
int n;
{
  int l,r,s,m,i,j,temp;

  s=1;
  sl[s]=1;
  sr[s]=n;

  do {
     l=sl[s];
     r=sr[s];
     s--;
    do { 
     i=l;
     j=r;
     m=f[(l+r)/2];
     do {
        while (f[i] < m) {
           i++;
        } /* while */
        while (f[j] > m) {
           j--;
        } /* while */
        if (i <= j) {
           temp=f[i];
           f[i]=f[j];
           f[j]=temp;
           i++;
           j--;
        } /* if */
     } while (i <= j);
     if (i < r) {
        s++;
        sl[s]=i;
        sr[s]=r;
     } /* if */
     r=j;
    } while (l < r);
  } while (s > 0);
   
} /* qqsort */


int main(argc,argv) 
int argc;
char *argv[];

{
   struct image_head header,header2;    
   short **image;
   short **im;
  
   int i,j,k,l,m,s,n,width,height,order,lim;
   int ai,bi,aj,bj,oo;

   int *v;
   int *sr,*sl;

   clock_t time1,timediff;


   if (argc<5) {
      printf("Usage: %s inimage outimage orderi limiti\n",argv[0]);
      exit(0);
   } /* if */

   ReadHeader(argv[1],&header);

   image = (short **)calloc(header.hgh,sizeof(short*));

   ReadImage(argv[1],image);


   header2.id=image_magic;
   header2.typ=im_s16;
   header2.off=512;
   header2.hgh=header.hgh;
   header2.wid=header.wid;

   im = (short **)calloc(header2.hgh,sizeof(short*));

   CreateIm_s16(im,header2.hgh,header2.wid);


order=atoi(argv[3]);
lim=atoi(argv[4]);


oo=(int)(log((double)(order*order))/log(2.0))+11;

v = (int *)calloc(order*order+1,sizeof(int));
sr = (int *)calloc(oo+1,sizeof(int));
sl = (int *)calloc(oo+1,sizeof(int));

width=header.wid;
height=header.hgh;

for (i=0; i<width; i++) {
   for (j=0; j<height; j++) {
      im[j][i]=image[j][i];
   } /* for j */
} /* for i */

s=(order-1)/2;

time1=clock();

for (i=0; i<width; i++) {
   ai=s;
   bi=s;
   if (i<s) {
      ai=i;
   } /* if */
   if (i>(width-1-s)) {
      bi=width-1-i;
   } /* if */
   for (j=0; j<height; j++) {
      aj=s;
      bj=s;
      if (j<s) {
         aj=j;
      } /* if */
      if (j>(height-1-s)) {
         bj=height-1-j;
      } /* if */
      m=1;
      for (k=-ai; k<=bi; k++) {
         for (l=-aj; l<=bj; l++) {
            v[m]=im[j+l][i+k];
            m++;
         } /* for l */
      } /* for k */
      qqsort(v,sr,sl,m-1);
      if (abs(image[j][i]-v[m/2]) > lim) {
         image[j][i]=v[m/2];
      }
    } /* for j */
} /* for i */


timediff=clock()-time1;
printf("%10.2f\n",(double)(timediff)/timetic);

WriteImage(argv[2],image,header,im_s16);

} /* End  */
