/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*********************************************************************/
/*  myrwimage.c -  routine to read and write GOP images (u8 and s16) */
/*                 to and from C-code. Dynamical allocation.         */ 
/*                                        latest update: AJO NOV 90  */
/*                                                        SS DEC 92  */
/*********************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "image.h" 

/*  # READ HEADER ################################################ */

void ReadHeader(NameOfFile, head_ptr)
char *NameOfFile;
struct image_head *head_ptr;
{
   FILE *fp;
   int i,ndir;
   char NameOfFile2[80];
   char env[256];
   char *dir[33];
   

   strcpy(env,getenv("IMGPATH"));


     sprintf(NameOfFile2,"%s/%s.imf",env,NameOfFile); 

     if ((fp = fopen(NameOfFile2,"rb")) == NULL) { 
        printf("Unable to open GOP file.\n");
        printf("Program aborted.\n");
        exit(0);
     } /* if */

     fread(head_ptr,sizeof(struct image_head),1,fp);
     
     fclose(fp);

}  /* End ReadHeader */


/*  # READ PIXELS ################################################# */

void ReadImage(NameOfFile, image)
char           *NameOfFile;
void          **image;
{
   FILE *fp;
   short row, col;
   int   i,ndir;
   short *shortptr;
   unsigned char  *charptr;
   void          *ptr;
   struct image_head header[3];
   char NameOfFile2[80];
   char env[256];


     strcpy(env,getenv("IMGPATH"));

     sprintf(NameOfFile2,"%s/%s.imf",env,NameOfFile); 

     if ((fp = fopen(NameOfFile2,"rb")) == NULL) { 
        printf("Unable to open GOP file.\n");
        printf("Program aborted.\n");
        exit(0);
     } /* if */


   /* READ HEADER */

   fp = fopen(NameOfFile2,"rb"); 

   fread(&header[1],sizeof(struct image_head),1,fp);



   /* DYNAMICAL ALLOCATION OF PIXEL SPACE */

   if (header[1].typ == im_s16) {
      shortptr = (short *)malloc(header[1].len);
      fseek(fp, header[1].off, SEEK_SET); 
      fread(shortptr,sizeof(short),header[1].len,fp); 
   }
   else {
      charptr = (unsigned char *)malloc(header[1].len);
      fseek(fp, header[1].off, SEEK_SET); 
      fread(charptr,sizeof(unsigned char),header[1].len,fp); 
   } /* if */
  
   fclose(fp); 

   if (header[1].typ == im_s16) {
      for ( row=0; row<header[1].hgh; row++ ) {
         image[row] = shortptr; 
         shortptr += header[1].wid;
      } /* for row */
   }
   else {
      for ( row=0; row<header[1].hgh; row++ ) {
         image[row] = charptr; 
         charptr += header[1].wid;
      } /* for row */
   } /* if */

}  /* End ReadImage */


/*  # WRITE PIXELS ################################################ */

void WriteImage(NameOfFile, image, header,imtyp)
char           *NameOfFile;
void          **image;
struct image_head header;
unsigned short imtyp;
{
   FILE *fp;
   short row, col;
   int   i,ndir;
   short index;
   char  temp_byte[3], NameOfFile2[80];
   char env[256];


   strcpy(env,getenv("IMGPATH"));

   /* SELECT LENGTH */

   header.typ = imtyp;

   if (header.typ == im_s16) {
      header.len = 2*(header.wid*header.hgh);
   }
   else {
      header.len = header.wid*header.hgh;
   } /* if */

   /* WRITE HEADER */

   sprintf(NameOfFile2,"%s/%s.imf",env,NameOfFile); 

   fp = fopen(NameOfFile2,"wb");

   fwrite(&header,sizeof(struct image_head),1,fp);


   /* FILL HEADER SPACE */

   temp_byte[1] = 0;

   for ( index=0; index<(header.off-sizeof(struct image_head)); index++ ) {
      fwrite(&temp_byte[1],sizeof(char),1,fp);
   } /* for index */

 
   /* WRITE IMAGE DATA */

   if (header.typ == im_s16) {
      for ( row=0; row<header.hgh; row++ ) {
	 fwrite(image[row],sizeof(short),header.wid,fp);
      } /* for row */
   }
   else {
      for ( row=0; row<header.hgh; row++ ) {
	 fwrite(image[row],sizeof(char),header.wid,fp);
      } /* for row */
   } /* if */

   fclose(fp);

}  /* End WriteImage */
