/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

#include <math.h>

void ccass(a,b,le)
float a[];
float b[];
int le;
{  
   int i,le2;

   le2=2*le;
 
   for (i=1; i<=le2; i=i+2) {
      a[i]=b[i];
      a[i+1]=b[i+1];
   }
} /* ccass */

void crass(a,b,le)
float a[];
float b[];
int le;
{
   int i,j,le2;

   le2=2*le;

   j=1;
   for (i=1; i<=le2; i=i+2) {
      a[i]=b[j];
      a[i+1]=0.0;
      j++;
   }
} /* crass */

void rcass(a,b,le)
float a[];
float b[];
int le;
{
   int i,j,le2;

   le2=2*le;

   j=1;
   for (i=1; i<=le2; i=i+2) {
      a[j]=b[i];
      j++;
   }
} /* rcass */

void rrass(a,b,le)
float a[];
float b[];
int le;
{
   int i;

   for (i=1; i<=le; i++) {
      a[i]=b[i];
   }
} /* rrass */



void ccpadd(a,b,le,re,im)
float a[];
float b[];
int le; 
double re,im;   
{
  int i,le2;

  le2=2*le;

  for (i=1; i<=le2; i=i+2) {
     a[i]=b[i]+re;
     a[i+1]=b[i+1]+im;
  }
} /* ccpadd */

void ccpmul(a,b,le,re,im)
float a[];
float b[];
int le;
double re,im;
{
   int i,le2;
   double tempr,tempi;

   le2=2*le;

   for (i=1; i<=le2; i=i+2) {
      tempr=re*b[i]-im*b[i+1];
      tempi=re*b[i+1]+im*b[i];
      a[i]=tempr;
      a[i+1]=tempi;
   }
} /* ccpmul */


void ccpdiv(a,b,le,re,im)
float a[];
float b[];
int le;
double re,im;
{
   int i,le2;
   double tempr,tempi,tempn;

   le2=2*le;

   for (i=1; i<=le2; i=i+2) {
      tempn=re*re+im*im;
      tempr=re*b[i]+im*b[i+1];
      tempi=re*b[i+1]-im*b[i];
      a[i]=tempr/tempn;
      a[i+1]=tempi/tempn;
   }
} /* ccpdiv */



void cccadd(a,b,c,le,re,im)
float a[];
float b[];
float c[];
int le;
double re,im;
{
  int i,le2;

  le2=2*le;

  for (i=1; i<=le2; i=i+2) {
     a[i]=b[i]+c[i]+re;
     a[i+1]=b[i+1]+c[i+1]+im;
  }
} /* cccadd */

void cccsub(a,b,c,le,re,im)
float a[];
float b[];
float c[];
int le;
double re,im;
{
  int i,le2;
  
  le2=2*le;

  for (i=1; i<=le2; i=i+2) {
     a[i]=b[i]-c[i]+re;
     a[i+1]=b[i+1]-c[i+1]+im;
  }
} /* cccsub */


void cccmul(a,b,c,le,amp,sgn)
float a[];
float b[];
float c[];
int le;
double amp;
double sgn;
{
  int i,le2;
  double r1,i1,r2,i2;
  
  le2=2*le;

  for (i=1; i<=le2; i=i+2) {
     r1=b[i];
     i1=b[i+1];
     r2=c[i];
     i2=sgn*c[i+1];
     a[i]=amp*(r1*r2-i1*i2);
     a[i+1]=amp*(r1*i2+i1*r2);
  }
} /* cccmul */

void rrcmul(a,b,c,le,amp)
float a[];
float b[];
float c[];
int le;
double amp;
{
  int i,j,le2;
  
  le2=2*le;

  j=1;
  for (i=1; i<=le2; i=i+2) {
     a[j]=amp*b[j]*c[i];
     j++;
  }
} /* rrcmul */


void cccdiv(a,b,c,le,amp,eps)
float a[];
float b[];
float c[];
int le;
double amp,eps;
{
  int i,le2;
  double tempn,tempr,tempi;
  
  le2=2*le;

  for (i=1; i<=le2; i=i+2) {
     tempn=c[i]*c[i]+c[i+1]*c[i+1];
     if (tempn==0.0) {
        tempn=eps;
     }
     tempr=b[i]*c[i]+b[i+1]+c[i+1];
     tempi=b[i+1]*c[i]-b[i]*c[i+1];
     a[i]=amp*tempr/tempn;
     a[i+1]=amp*tempi/tempn;
  }
} /* cccdiv */

void crcdiv(a,b,c,le,amp,eps)
float a[];
float b[];
float c[];
int le;
double amp,eps;
{
  int i,j,le2;
  double tempn;
  
  le2=2*le;

  j=1;
  for (i=1; i<=le2; i=i+2) {
     tempn=c[i];
     if (tempn==0.0) {
        tempn=eps;
     }
     a[i]=amp*b[j]/tempn;
     a[i+1]=0.0;
     j++;
  }
} /* crcdiv */
