/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*********************************************************************/
/*  allocate - routine to read and write GOP images (u8 and s16)     */
/*                 to and from C-code. Dynamical allocation.         */ 
/*                                        latest update: AJO NOV 90  */
/*********************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "image.h" 

/*  # UNSIGNED 8 ################################################# */

void CreateIm_u8(image,rows,cols)
short           rows, cols; 
unsigned char   **image;
{
   short row;
   unsigned char  *charptr;

   /* DYNAMICAL ALLOCATION OF PIXEL SPACE */

   charptr = (unsigned char *)calloc(cols*rows,sizeof(unsigned char));

   for ( row=0; row<rows; row++ ) {
      image[row] = charptr; 
      charptr += cols;
   } /* for row */

}  /* End CreateIm_u8 */

/*  # SIGNED 16 ################################################# */

void CreateIm_s16(image,rows,cols)
short           rows, cols; 
short           **image;
{
   short row;
   short *shortptr;

   /* DYNAMICAL ALLOCATION OF PIXEL SPACE */

   shortptr = (short *)calloc(cols*rows,sizeof(short));

   for ( row=0; row<rows; row++ ) {
      image[row] = shortptr; 
      shortptr += cols;
   } /* for row */

}  /* End CreateIm_s16 */

/*  # FLOATING ################################################# */

void CreateIm_fl(image,rows,cols)
short           rows, cols; 
float           **image;
{
   short row;
   float  *floatptr;

   /* DYNAMICAL ALLOCATION OF PIXEL SPACE */

   floatptr = (float *)calloc(cols*rows,sizeof(float));

   for ( row=0; row<rows; row++ ) {
      image[row] = floatptr; 
      floatptr += cols;
   } /* for row */

}  /* End CreateIm_fl */

