/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  imcomb.c   SS June 98                                    */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "image.h"


int main(argc,argv) 
int argc;
char *argv[];
{
   struct image_head inheader[11][11]; 
   struct image_head header; 

   short **inimage[11][11];
   short **image;
   
   int i,j,i2,j2,hgh,wid,edge;
   int nrow,ncol;

   if (argc<5) {
      printf("Usage: %s nrowi ncoli\n",argv[0]);
      printf("            im_11 im_12   ... im_1nc\n");
      printf("            im_21 im_22   ... im_2nc\n");
      printf("             ...   ...    ...  ...\n");
      printf("            im_nr1 im_nr2 ... im_nrnc\n");
      printf("            outimage edgei\n");
      exit(0);
   } /* if */

   nrow=atoi(argv[1]);
   ncol=atoi(argv[2]);

 
   for (i=1; i<=nrow; i++) {
      for (j=1; j<=ncol; j++) {
         ReadHeader(argv[ncol*(i-1)+j+2],&inheader[i][j]);
         inimage[i][j] = (short **)calloc(inheader[i][j].hgh,sizeof(short*));
         ReadImage(argv[ncol*(i-1)+j+2],inimage[i][j]);
      } /* for j */
   } /* for i */


   edge=atoi(argv[ncol*nrow+4]);

   header.id=image_magic;
   header.typ=im_s16;
   header.off=512;
   header.hgh=nrow*inheader[1][1].hgh+(nrow-1)*edge;
   header.wid=ncol*inheader[1][1].wid+(ncol-1)*edge;

   image = (short **)calloc(header.hgh,sizeof(short*));

   CreateIm_s16(image,header.hgh,header.wid);

   for (i=0; i<header.hgh; i++) {
       for (j=0; j<header.wid; j++) {
           image[i][j]=32767;
       } /* for j */
   } /* for i */


   hgh=inheader[1][1].hgh;
   wid=inheader[1][1].wid;

   for (i2=0; i2<nrow; i2++) {
      for (j2=0; j2<ncol; j2++) {
         for (i=0; i<wid; i++) {
            for (j=0; j<hgh; j++) {
                image[j+i2*hgh+i2*edge][i+j2*wid+j2*edge]=inimage[i2+1][j2+1][j][i];
            } /* for j */
         } /* for i */
      } /* for j2 */
   } /* for i2 */


   WriteImage(argv[ncol*nrow+3],image,header,im_s16);

} /* End  */
